% Matlab script that reads the sqrt(PSD) generated by spectre and uses
% Carlos' routine to find the appropriate filter.

load in.cl;
infr = in(:,1);
inno = in(:,2);
period = 80e-9;
Np = 14;
Nz = 14;

infr(length(infr)) = (1.0-1.0e-5)*infr(length(infr));
% Unwrap the frequency vector
F = (1/2/pi)*(2/period)*tan(infr*2*pi*period/2);
S = inno.^2;

[B2,A2]=l2minity(S,2*pi*F,Nz,Np);

% Plot the poles and the zeros 

figure(8);
Sest = abs(freqs(B2, A2, 2*pi*F));
Fp = abs(roots(A2))/2/pi;
Sp = abs(freqs(B2,A2,2*pi*Fp));
Fz = abs(roots(B2))/2/pi;
Sz = abs(freqs(B2,A2,2*pi*Fz));
loglog(F,S,'.',   F, Sest,   Fp,Sp,'xr',   Fz,Sz,'og');

poles2 = roots(A2);
zeros2 = roots(B2);

% Keep only poles and zeros in the left half plane

k=1;
for i=1:length(poles2),
  if (real(poles2(i)) <= 0) s_poles(k) = poles2(i); k=k+1; end;
end;

k=1;
for i=1:length(zeros2),
  if (real(zeros2(i)) <= 0) s_zeros(k) = zeros2(i); k=k+1; end;
end;

% Pass to the z domain
[z_poles, z_zeros ] = s2z (s_poles, s_zeros, period);


% Change from pole-zero form to pole residue form
% Calculate filter gain for magnitude MagGain at frequency FregGain.
MagGain = inno(1)^2;
FregGain = infr(1);
[z_rpoles, rres, z_cpoles, cres, gain ] = pz2pr(z_poles, z_zeros, FregGain, MagGain, period);

% Plot what we found
F = logspace (-2, 7.5, 2000);
[H, Szz] = psdzac(F*period, z_rpoles, z_cpoles, rres, cres);
Szz = gain^2 * Szz;

[Hz1, Szz1] = psdzac([1e-2]*period, z_rpoles, z_cpoles, rres, cres);
dcgainZ = gain * abs(Hz1);
[Hs1, Sss1] = psdac([1e-2], s_poles, s_zeros);
dcgainS = 1/abs(Hs1) * dcgainZ;
[Hs, Sss] = psdac(F, s_poles, s_zeros );
Sss = dcgainS^2 * Sss;

figure(9);
% loglog(F, Szz, '--', F, Sss, '-', infr, inno.^2 ,'.');
loglog(F, Szz, '--', infr, inno.^2 ,'o');
xlabel('Frequency (Hz)');
ylabel('Input referred noise (V^2/Hz)');
print -depsc 'psdfit.eps'

% print the coefficients of the discrete time filter, 
% to be copied to the verilog file where the filter is implemented
z_rpoles
rres
z_cpoles
cres
gain
